#include <formatio.h>
#include "rsvna.h"
#include <cvirte.h>     
#include <userint.h>
#include <visa.h>
#include "rsvna_assigning_channels_traces_diagrams_example.h"


#define ARRAY_SIZE    5000
#define STRING_LENGTH 512
#define WINDOW        1

/* Useful macros */
#define CHECKERR(fCal) \
    if (status = checkError((fCal)), status < VI_SUCCESS) \
        return status; else


/* Global variables */
static   ViInt32    panelHandle  = 0;
static   ViSession  instrSession = VI_NULL;
    

/* Utility functions declarations */
ViStatus checkError(ViStatus status);
void cleanup (ViBoolean deviceClear);
void EnableButtons(ViBoolean enable);

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)
        return -1;  /* out of memory */
    if ((panelHandle = LoadPanel (0, "rsvna_assigning_channels_traces_diagrams_example.uir", PANEL)) < 0)
        return -1;
    
    DisplayPanel (panelHandle);
    RunUserInterface ();
    DiscardPanel (panelHandle);
    
    return 0;
}

int CVICALLBACK panelCallback (int panel, int event, void *callbackData,
        int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_CLOSE:
            /* cleanup specan driver session */
            cleanup(VI_FALSE);
            
            QuitUserInterface (0);
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Start Button Callback
/* Purpose:  Configures the instrument and initiates the measurement
/*===========================================================================*/
int CVICALLBACK startButton (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViChar      instr_desc[STRING_LENGTH] = "";
    ViInt32     id               = 0;
    ViInt32     reset            = VI_FALSE;
    ViInt32     display          = VI_FALSE;
    ViStatus    status           = VI_SUCCESS;
    ViInt32     trace            = 0;
    ViInt32     buffer_size      = STRING_LENGTH;
    ViChar      catalog[STRING_LENGTH] = "";
    ViInt32     channel          = 0;
    ViChar      trace_name[STRING_LENGTH] = "";
    ViInt32     S_out_port       = 0;
    ViInt32     S_in_port        = 0;
    ViInt32     window           = 0;
    ViInt32     trace_format     = 0;
    
    switch (event)
    {
        case EVENT_COMMIT:     /* start button pressed */
        {
        
            /* block mouse events */
            SetWaitCursor (1);
            
            /* read values from panel*/
            GetCtrlVal (panelHandle, PANEL_INSTR_DESC, instr_desc);
            GetCtrlVal (panelHandle, PANEL_IDQUERY, &id);
            GetCtrlVal (panelHandle, PANEL_RESET, &reset);
            GetCtrlVal (panelHandle, PANEL_DISPLAYON, &display);
            GetCtrlVal (panelHandle, PANEL_CHANNEL, &channel);
            GetCtrlVal (panelHandle, PANEL_TRACE_NAME, trace_name);
            GetCtrlVal (panelHandle, PANEL_S_OUT_PORT, &S_out_port);
            GetCtrlVal (panelHandle, PANEL_S_IN_PORT, &S_in_port);
            GetCtrlVal (panelHandle, PANEL_WINDOW, &window);
            GetCtrlVal (panelHandle, PANEL_FORMAT, &trace_format);
            
            /* disable buttons for configuration */
            EnableButtons(VI_FALSE);
            
            /* initialize driver */
            CHECKERR (rsvna_init (instr_desc, id, reset, &instrSession));
            
            /* Clean front panel textbox */ 
            DefaultCtrl (panelHandle, PANEL_CATALOG);
            
            /* switch device display on/off */
            CHECKERR( rsvna_SetAttributeViInt32 (instrSession, "", RSVNA_ATTR_DISPLAY_UPDATE, display));
            
            /* Add a new channel if it doesn't exist */
            CHECKERR (rsvna_ChannelAdd (instrSession, channel, ""));
            
            /* Create a new trace */
            CHECKERR (rsvna_SelectSParameters (instrSession, channel, trace_name, S_out_port, S_in_port));
            
            /* Select active trace */
            CHECKERR (rsvna_TraceSelect (instrSession, channel, trace_name));
            
            /* Change the format for trace */
            CHECKERR (rsvna_ConfigureTraceFormat (instrSession, channel, trace_format, 10));
            
            /* Display the trace in the display area */
            CHECKERR (rsvna_TraceAssignDiagramArea (instrSession, window, trace_name));

            /* List the traces, assigned to a certain Channel */
            CHECKERR (rsvna_TraceList (instrSession, channel, catalog, buffer_size));
            
            /* Show trace list in front panel textbox */ 
            SetCtrlVal (panelHandle, PANEL_CATALOG, catalog);
            
            MessagePopup ("Vector Network Analyzer Example", "You should be able to see the added trace on the instrument now. Press OK to continue.");
            
            /* Find the number for the trace */
            CHECKERR (rsvna_QueryTraceNumber (instrSession, trace_name, &trace));
            
            /* Remove the trace from the diagram area */
            CHECKERR (rsvna_TraceUnassignDiagramArea (instrSession, window, trace));

            /* Delete a trace with a specified trace name and channel. */
            CHECKERR (rsvna_TraceDelete (instrSession, channel, trace_name));
            
            /* close session */
            cleanup(VI_FALSE);
        }
        break;
    }
    return 0;
}


/*===========================================================================*/
/* Function: checkError
/* Purpose:  If passed status indicates an failure, the error message will
/*           be displayed.
/*===========================================================================*/
ViStatus checkError (ViStatus status)
{
    ViChar  error_message [STRING_LENGTH];
    ViChar  error_buffer  [STRING_LENGTH*2];
    ViChar* p2buf;
    ViInt32 error;
    
    if (status < VI_SUCCESS)
    {
        /* Converts a status code returned by an instrument driver function into a user-readable string */
        rsvna_error_message (instrSession, status, error_message);
        p2buf = error_buffer + sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", status, error_message);
        
        
        /* This function reads an error code and a message from the instrument's error queue */
        rsvna_error_query (instrSession, &error, error_message);
        
        SetWaitCursor (0);
        if (error != VI_SUCCESS)
        {
            sprintf (p2buf, "Secondary Error: 0x%08X, %s\n", error, error_message);
        }
        MessagePopup ("Error", error_buffer);
        
        /* close session */
        cleanup(VI_TRUE);
        
        /* enable buttons for configuration */
        EnableButtons(VI_TRUE);
    }
    return status;
}


/*===========================================================================*/
/* Function: EnableButtons
/* Purpose:  This function dis-/enables GUI while measuring/configuration.
/*===========================================================================*/
void EnableButtons(ViBoolean enable)
{
    SetCtrlAttribute (panelHandle, PANEL_START,      ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_RESET,      ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_IDQUERY,    ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_INSTR_DESC, ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_DISPLAYON,  ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_CHANNEL,    ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_TRACE_NAME, ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_S_IN_PORT,  ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_S_OUT_PORT, ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_WINDOW,     ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_FORMAT,     ATTR_DIMMED, !enable);
}


/*===========================================================================*/
/* Function: Cleanup
/* Purpose:  Closes the VISA session to the instrument and in case of error
/*           queries the error and shows the dialog box with the passed error
/*           message. In case Device Clear is requested, it is necessary to 
/*           set the argument deviceClear to VI_TRUE.
/*===========================================================================*/
void cleanup (ViBoolean deviceClear)
{

    /* Device Clear is a low-level command and should be used in case the
     * instrument is waiting for Operation Complete to cancel the wait.
     * It is useful for instance in case of incorrect external trigger
     * when the instrument does not respond to any other command because of
     * waiting for trigger.
     */
    if (deviceClear)
    {
        viClear (instrSession);
        viPrintf (instrSession, "*CLS\n");
    }
    
    /* close instrument session */
    rsvna_close (instrSession);
    instrSession = VI_NULL;
    
    /* enable GUI */
    EnableButtons (1);
    /* enable mouse events*/
    SetWaitCursor (0);
}


