#include "rsvna.h"
#include <cvirte.h>     
#include <userint.h>
#include <visa.h>
#include "rsvna_marker_search_example.h"


#define ARRAY_SIZE    5000
#define STRING_LENGTH 512

/* Useful macros */
#define CHECKERR(fCal) \
    if (status = checkError((fCal)), status < VI_SUCCESS) \
        return status; else


/* Global variables */
static   ViInt32    panelHandle  = 0;
static   ViSession  instrSession = VI_NULL;
    

/* Utility functions declarations */
ViStatus checkError(ViStatus status);
void cleanup (ViBoolean deviceClear);
void EnableButtons(ViBoolean enable);
int readTrace(void);

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)
        return -1;  /* out of memory */
    if ((panelHandle = LoadPanel (0, "rsvna_marker_search_example.uir", PANEL)) < 0)
        return -1;
    
    DisplayPanel (panelHandle);
    RunUserInterface ();
    DiscardPanel (panelHandle);
    
    return 0;
}

int CVICALLBACK panelCallback (int panel, int event, void *callbackData,
        int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_CLOSE:
            /* cleanup specan driver session */
            cleanup(VI_FALSE);
            
            QuitUserInterface (0);
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Start Button Callback
/* Purpose:  Configures the instrument and initiates the measurement
/*===========================================================================*/
int CVICALLBACK startButton (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViChar      instr_desc [STRING_LENGTH] = "";
    ViInt32     id               = 0;
    ViInt32     reset            = VI_FALSE;
    ViInt32     display          = VI_FALSE;
    ViInt32     peakType         = 0;
    ViInt32     input            = 0;
    ViInt32     marker           = 0;
    ViStatus    status           = VI_SUCCESS;
    ViReal64    horizontalPos    = 0.0;
    ViReal64    amplitude[2];
    ViChar      rep_cap [STRING_LENGTH] = "";
    
    switch (event)
    {
        case EVENT_COMMIT:     /* start button pressed */
        {
        
            /* block mouse events */
            SetWaitCursor (1);
            
            /* read values from panel*/
            GetCtrlVal (panelHandle, PANEL_INSTR_DESC, instr_desc);
            GetCtrlVal (panelHandle, PANEL_IDQUERY, &id);
            GetCtrlVal (panelHandle, PANEL_RESET, &reset);
            GetCtrlVal (panelHandle, PANEL_DISPLAYON, &display);
            GetCtrlVal (panelHandle, PANEL_PEAKTYPE, &peakType);
            GetCtrlVal (panelHandle, PANEL_MARKER, &marker);

            /* disable buttons for configuration */
            EnableButtons(VI_FALSE);
            
            /* initialize driver */
            CHECKERR (rsvna_init (instr_desc, id, reset, &instrSession));
            
            /* switch device display on/off */
            CHECKERR (rsvna_SetAttributeViInt32 (instrSession, "", RSVNA_ATTR_DISPLAY_UPDATE, display));
            
            /* Set instrument to single sweep mode - continous mode is done
             * programmatically because this is the only way how to
             * synchronize measurement using *OPC
             */
            CHECKERR (rsvna_ConfigureAcquisition (instrSession, 1, RSVNA_VAL_SWEEP_SINGLE_CHAN, VI_FALSE, 10, RSVNA_VAL_SWEEP_ALL_CHAN));
            
            sprintf (rep_cap, "Channel1,Mk%ld", marker);
            
            /* Enable marker. */
            CHECKERR (rsvna_SetAttributeViBoolean (instrSession, rep_cap, RSVNA_ATTR_MARKER_STATE, VI_TRUE));
            
            /* Perform Sweep */
            CHECKERR (rsvna_RestartSweep (instrSession, 1, RSVNA_VAL_SWEEP_ALL_CHAN));
        
            /* specifies the type of marker search and performs the search */
            CHECKERR (rsvna_MarkerSearch (instrSession, 1, marker, peakType));
        
            /* returns the position of the selected marker */
            CHECKERR (rsvna_GetAttributeViReal64 (instrSession, rep_cap, RSVNA_ATTR_MARKER_STIMULUS, &horizontalPos));
            CHECKERR (rsvna_QueryMarkerResponse (instrSession, 1, marker, amplitude));
       
            /* write marker position to panel */
            SetCtrlVal(panelHandle, PANEL_MARKERPOS, horizontalPos);
            SetCtrlVal(panelHandle, PANEL_MARKERAMP_X, amplitude[0]);
            SetCtrlVal(panelHandle, PANEL_MARKERAMP_Y, amplitude[1]);
            
            /* enable mouse events*/
            SetWaitCursor (0);
        }
        break;
    }
    return 0;
}



/*===========================================================================*/
/* Function: Stop Button Callback
/* Purpose:  Stops the continuous measurement, in other cases has no sense
/*===========================================================================*/
int CVICALLBACK stopButton (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
         
    {
        case EVENT_COMMIT:
            
            /* cleanup specan driver session */
            cleanup(VI_TRUE);
                
            /* enable buttons for configuration */
            EnableButtons(VI_TRUE);
            
            
            break;
    }
    return 0;
}


/*===========================================================================*/
/* Function: Search Button Callback
/* Purpose:  Sets peak excursion value, reads position of selected marker
/*           and writes marker position to panel
/*===========================================================================*/
 int  CVICALLBACK searchButton(int panel, int control, int event,
     void *callbackData, int eventData1, int eventData2)
{ 
    ViStatus  temp_status  = VI_SUCCESS;
    ViStatus  status       = VI_SUCCESS;
    ViInt32   peakType     = 0;
    ViInt32   marker       = 0;
    ViReal64  horizontalPos = 0.0;
    ViReal64  amplitude[2];
    ViChar    rep_cap [STRING_LENGTH] = "";
    
    switch (event)
    {
        case EVENT_COMMIT:
         
     
            /* read peak type value from panel */
            GetCtrlVal (panelHandle, PANEL_PEAKTYPE, &peakType);
            GetCtrlVal (panelHandle, PANEL_MARKER, &marker);
            
            sprintf (rep_cap, "Channel1,Mk%ld", marker);
            
            /* Enable marker. */
            CHECKERR (rsvna_SetAttributeViBoolean (instrSession, rep_cap, RSVNA_ATTR_MARKER_STATE, VI_TRUE));
            
            CHECKERR (rsvna_MarkerSearch (instrSession, 1, marker, peakType));
            
            /* returns the position of the selected marker */
            CHECKERR (rsvna_GetAttributeViReal64 (instrSession, rep_cap, RSVNA_ATTR_MARKER_STIMULUS, &horizontalPos));
            CHECKERR (rsvna_QueryMarkerResponse (instrSession, 1, marker, amplitude));
           
            /* write marker position to panel */
            SetCtrlVal(panelHandle, PANEL_MARKERPOS, horizontalPos);
            SetCtrlVal(panelHandle, PANEL_MARKERAMP_X, amplitude[0]);
            SetCtrlVal(panelHandle, PANEL_MARKERAMP_Y, amplitude[1]);
            
            break;
    }
    return 0;
}



/*===========================================================================*/
/* Function: checkError
/* Purpose:  If passed status indicates an failure, the error message will
/*           be displayed.
/*===========================================================================*/
ViStatus checkError (ViStatus status)
{
    ViChar  error_message [STRING_LENGTH];
    ViChar  error_buffer  [STRING_LENGTH*2];
    ViChar* p2buf;
    ViInt32 error;
    
    if (status < VI_SUCCESS)
    {
        /* Converts a status code returned by an instrument driver function into a user-readable string */
        rsvna_error_message (instrSession, status, error_message);
        p2buf = error_buffer + sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", status, error_message);
        
        
        /* This function reads an error code and a message from the instrument's error queue */
        rsvna_error_query (instrSession, &error, error_message);
        
        SetWaitCursor (0);
        if (error != VI_SUCCESS)
        {
            sprintf (p2buf, "Secondary Error: 0x%08X, %s\n", error, error_message);
        }
        MessagePopup ("Error", error_buffer);
        
        /* close session */
        cleanup(VI_TRUE);
        
        /* enable buttons for configuration */
        EnableButtons(VI_TRUE);
    }
    return status;
}


/*===========================================================================*/
/* Function: EnableButtons
/* Purpose:  This function dis-/enables GUI while measuring/configuration.
/*===========================================================================*/
void EnableButtons(ViBoolean enable){

    SetCtrlAttribute (panelHandle, PANEL_START,     ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_RESET,     ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_IDQUERY,   ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_INSTR_DESC,ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_DISPLAYON, ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_MARKER,    ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_STOP,      ATTR_DIMMED, enable);
    SetCtrlAttribute (panelHandle, PANEL_SEARCH,    ATTR_DIMMED, enable);
    SetCtrlAttribute (panelHandle, PANEL_PEAKTYPE,  ATTR_DIMMED, enable);
    SetCtrlAttribute (panelHandle, PANEL_MARKERPOS, ATTR_DIMMED, enable);
    SetCtrlAttribute (panelHandle, PANEL_MARKERAMP_X, ATTR_DIMMED, enable);
    SetCtrlAttribute (panelHandle, PANEL_MARKERAMP_Y, ATTR_DIMMED, enable);
}


/*===========================================================================*/
/* Function: Cleanup
/* Purpose:  Closes the VISA session to the instrument and in case of error
/*           queries the error and shows the dialog box with the passed error
/*           message. In case Device Clear is requested, it is necessary to 
/*           set the argument deviceClear to VI_TRUE.
/*===========================================================================*/
void cleanup (ViBoolean deviceClear)
{

    /* Device Clear is a low-level command and should be used in case the
     * instrument is waiting for Operation Complete to cancel the wait.
     * It is useful for instance in case of incorrect external trigger
     * when the instrument does not respond to any other command because of
     * waiting for trigger.
     */
    if (deviceClear)
    {
        viClear (instrSession);
        viPrintf (instrSession, "*CLS\n");
    }
    
    /* close instrument session */
    rsvna_close (instrSession);
    instrSession = VI_NULL;
    
    
    SetCtrlIndex (panelHandle, PANEL_PEAKTYPE, 0);
    
    /* enable GUI */
    EnableButtons (1);
    /* enable mouse events*/
    SetWaitCursor (0);
    
}



            
