#include "rsvna.h"
#include <cvirte.h>     
#include <userint.h>
#include <visa.h>
#include "rsvna_saving_and_recalling_error_terms_example.h"


#define ARRAY_SIZE    5000
#define STRING_LENGTH 512
#define WINDOW        1
#define CR 13            /* Decimal code of Carriage Return char */
#define LF 10            /* Decimal code of Line Feed char */

/* Useful macros */
#define CHECKERR(fCal) \
    if (status = checkError((fCal)), status < VI_SUCCESS) \
        return status; else


/* Global variables */
static   ViInt32    panelHandle  = 0;
volatile ViBoolean  enableTrace  = VI_FALSE; 
static   ViSession  instrSession = VI_NULL;
ViReal64            amplitude [10][ARRAY_SIZE];
ViInt32             values_returned[10];

/* Utility functions declarations */
ViStatus checkError(ViStatus status);
void cleanup (ViBoolean deviceClear);
void EnableButtons(ViBoolean enable);

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)
        return -1;  /* out of memory */
    if ((panelHandle = LoadPanel (0, "rsvna_saving_and_recalling_error_terms_example.uir", PANEL)) < 0)
        return -1;
    
    DisplayPanel (panelHandle);
    RunUserInterface ();
    DiscardPanel (panelHandle);
    
    return 0;
}

int CVICALLBACK panelCallback (int panel, int event, void *callbackData,
        int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_CLOSE:
            /* cleanup specan driver session */
            cleanup(VI_FALSE);
            
            QuitUserInterface (0);
            break;
    }
    return 0;
}


/*===========================================================================*/
/* Function: Start Button Callback
/* Purpose:  Configures the instrument according to selected measurement
/*           parameters and enables the measurement loop
/*===========================================================================*/
int CVICALLBACK startButton (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViChar      instr_desc [STRING_LENGTH] = "";
    ViReal64    start_frequency  = 0.0;
    ViReal64    stop_frequency   = 0.0;
    ViInt32     number_of_points = 0;
    ViInt32     id               = 0;
    ViInt32     reset            = VI_FALSE;
    ViInt32     display          = VI_FALSE;
    ViStatus    status           = VI_SUCCESS;
    
    switch (event)
    {
        case EVENT_COMMIT:     /* start button pressed */
        {
        
            /* block mouse events */
            SetWaitCursor (1);
            
            /* read values from panel*/
            GetCtrlVal (panelHandle, PANEL_INSTR_DESC, instr_desc);
            GetCtrlVal (panelHandle, PANEL_IDQUERY, &id);
            GetCtrlVal (panelHandle, PANEL_RESET, &reset);
            GetCtrlVal (panelHandle, PANEL_START_FREQ, &start_frequency);
            GetCtrlVal (panelHandle, PANEL_STOP_FREQ, &stop_frequency);
            GetCtrlVal (panelHandle, PANEL_SWEEP_POINTS, &number_of_points);
            GetCtrlVal (panelHandle, PANEL_CHECKBOX, &display);

            /* disable buttons for configuration */
            EnableButtons(VI_FALSE);
            
            /* initialize driver */
            CHECKERR (rsvna_init (instr_desc, id, reset, &instrSession));
        
            /* switch device display on/off */
            CHECKERR (rsvna_SetAttributeViInt32 (instrSession, "", RSVNA_ATTR_DISPLAY_UPDATE, display));
        
            /* Set frequency range */
            CHECKERR (rsvna_ConfigureFrequencyStartStop (instrSession, 1, start_frequency, stop_frequency));
            
            /* Set number of sweep points */
            CHECKERR (rsvna_ConfigureSweepNumberOfPoints (instrSession, 1, number_of_points));

            /*  */
            CHECKERR (rsvna_SelectCalibrationType (instrSession, 1, "Test SFK TOSM 12", RSVNA_VAL_CALTYPE_TOSM, 1, 2, 0, 0));
                
            /* Measure Standards */
            MessagePopup ("Two-Port Calibration", "Connect/disconnect calibration unit for THROUGH calibration step.");
            CHECKERR (rsvna_StartCalibration (instrSession, 1, RSVNA_VAL_CALCOLLSTDGEN_THR, 1, 2));
            MessagePopup ("Two-Port Calibration", "Connect/disconnect calibration unit for OPEN port 1 calibration step.");
            CHECKERR (rsvna_StartCalibration (instrSession, 1, RSVNA_VAL_CALCOLLSTDGEN_OPEN, 1, 0));
            MessagePopup ("Two-Port Calibration", "Connect/disconnect calibration unit for SHORT port 1 calibration step.");
            CHECKERR (rsvna_StartCalibration (instrSession, 1, RSVNA_VAL_CALCOLLSTDGEN_SHORT, 1, 0));
            MessagePopup ("Two-Port Calibration", "Connect/disconnect calibration unit for MATCH port 1 calibration step.");
            CHECKERR (rsvna_StartCalibration (instrSession, 1, RSVNA_VAL_CALCOLLSTDGEN_MATCH, 1, 0));
            MessagePopup ("Two-Port Calibration", "Connect/disconnect calibration unit for OPEN port 2 calibration step.");
            CHECKERR (rsvna_StartCalibration (instrSession, 1, RSVNA_VAL_CALCOLLSTDGEN_OPEN, 2, 0));
            MessagePopup ("Two-Port Calibration", "Connect/disconnect calibration unit for SHORT port 2 calibration step.");
            CHECKERR (rsvna_StartCalibration (instrSession, 1, RSVNA_VAL_CALCOLLSTDGEN_SHORT, 2, 0));
            MessagePopup ("Two-Port Calibration", "Connect/disconnect calibration unit for MATCH port 2 calibration step.");
            CHECKERR (rsvna_StartCalibration (instrSession, 1, RSVNA_VAL_CALCOLLSTDGEN_MATCH, 2, 0));

            
            /* Apply calibration */
            CHECKERR (rsvna_SaveCalibrationData (instrSession, 1)); 
            
            /* Save error terms */
            CHECKERR (rsvna_TraceResponseDataError (instrSession, 1, 1, ARRAY_SIZE, &values_returned[0], amplitude[0]));
            CHECKERR (rsvna_TraceResponseDataError (instrSession, 1, 2, ARRAY_SIZE, &values_returned[1], amplitude[1]));
            CHECKERR (rsvna_TraceResponseDataError (instrSession, 1, 3, ARRAY_SIZE, &values_returned[2], amplitude[2]));
            CHECKERR (rsvna_TraceResponseDataError (instrSession, 1, 5, ARRAY_SIZE, &values_returned[3], amplitude[3]));
            CHECKERR (rsvna_TraceResponseDataError (instrSession, 1, 6, ARRAY_SIZE, &values_returned[4], amplitude[4]));
            CHECKERR (rsvna_TraceResponseDataError (instrSession, 1, 7, ARRAY_SIZE, &values_returned[5], amplitude[5]));
            CHECKERR (rsvna_TraceResponseDataError (instrSession, 1, 8, ARRAY_SIZE, &values_returned[6], amplitude[6]));
            CHECKERR (rsvna_TraceResponseDataError (instrSession, 1, 9, ARRAY_SIZE, &values_returned[7], amplitude[7]));
            CHECKERR (rsvna_TraceResponseDataError (instrSession, 1, 11, ARRAY_SIZE, &values_returned[8], amplitude[8]));
            CHECKERR (rsvna_TraceResponseDataError (instrSession, 1, 12, ARRAY_SIZE, &values_returned[9], amplitude[9]));
            
            /* start measurements */
            enableTrace = VI_TRUE;
            
            /* enable mouse events*/
            SetWaitCursor (0);
        }
        break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Stop Button Callback
/* Purpose:  Stops the continuous measurement, in other cases has no sense
/*===========================================================================*/
int CVICALLBACK stopButton (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
         
    {
        case EVENT_COMMIT:
            
            /* stop measurements */
            enableTrace = VI_FALSE;
            
            /* cleanup specan driver session */
            cleanup(VI_TRUE);
                
            /* enable buttons for configuration */
            EnableButtons(VI_TRUE);
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: plotErrorTerm Callback
/* Purpose:  Every time the Plot Error Term button is pressed the selected 
/*           error term data is plotted as a graph.
/*===========================================================================*/
int CVICALLBACK plotErrorTerm (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViStatus temp_status  = VI_SUCCESS;
    ViInt32  timeout      = 5000;               /* default value */
    ViChar   error_buffer [1024];
    ViInt32  error_term = 0;
    int i;
    
    switch (event)
    {
        case EVENT_COMMIT:  /* Plot Error Term button pressed */
            
            /* already configured and ready for measurements?*/
            if(enableTrace==VI_FALSE)
                return 0;
            
            /* exclusive execution from now on! */          
            enableTrace = VI_FALSE;

            GetCtrlVal (panelHandle, PANEL_ERROR_TERM, &error_term);
            
            if (error_term > 10)
            {
                MessagePopup ("Error Term", "There are only 10 error terms. Select error term small or equal to 10.");
                enableTrace = VI_TRUE;
                return -1;
            }
                
            /* deletes all points of the graph plot */
            temp_status = DeleteGraphPlot (panelHandle, PANEL_GRAPH, -1, VAL_DELAYED_DRAW);
            if( temp_status != 0 )
            {
                /* prompt CVI error message */
                MessagePopup ("Error", GetUILErrorString(temp_status));
                
                /* cleanup specan driver session */
                cleanup(VI_FALSE);
                
                /* do not free resource */
                enableTrace = VI_FALSE;
                return -1;
            }
            
            /* Plots a waveform onto a graph control */
            temp_status = PlotWaveform (panelHandle, PANEL_GRAPH, amplitude[error_term - 1], values_returned[error_term - 1], VAL_DOUBLE,
                            1.0, 0.0, 0.0, 1, VAL_THIN_LINE, VAL_SIMPLE_DOT,
                            VAL_SOLID, 1, VAL_YELLOW);
            if( temp_status < 0 )
            {
                /* prompt CVI error message */
                MessagePopup ("Error", GetUILErrorString(temp_status));
                
                /* cleanup specan driver session */
                cleanup(VI_FALSE);
                
                /* do not free resource */
                enableTrace = VI_FALSE;
                return -1;
            }
            
            /* free resource */         
            enableTrace = VI_TRUE;
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: checkError
/* Purpose:  If passed status indicates an failure, the error message will
/*           be displayed.
/*===========================================================================*/
ViStatus checkError (ViStatus status)
{
    ViChar  error_message [STRING_LENGTH];
    ViChar  error_buffer  [STRING_LENGTH*2];
    ViChar* p2buf;
    ViInt32 error;
    
    if (status < VI_SUCCESS)
    {
        /* Converts a status code returned by an instrument driver function into a user-readable string */
        rsvna_error_message (instrSession, status, error_message);
        p2buf = error_buffer + sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", status, error_message);
        
        
        /* This function reads an error code and a message from the instrument's error queue */
        rsvna_error_query (instrSession, &error, error_message);
        
        SetWaitCursor (0);
        if (error != VI_SUCCESS)
        {
            sprintf (p2buf, "Secondary Error: 0x%08X, %s\n", error, error_message);
        }
        MessagePopup ("Error", error_buffer);
        
        /* close session */
        cleanup(VI_TRUE);
        
        /* enable buttons for configuration */
        EnableButtons(VI_TRUE);
    }
    return status;
}


/*===========================================================================*/
/* Function: EnableButtons
/* Purpose:  This function dis-/enables GUI while measuring/configuration.
/*===========================================================================*/
void EnableButtons(ViBoolean enable){

    SetCtrlAttribute (panelHandle, PANEL_SWEEP_POINTS,     ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_START_FREQ,       ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_STOP_FREQ,        ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_START,            ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_RESET,            ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_IDQUERY,          ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_INSTR_DESC,       ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_CHECKBOX,         ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_STOP,             ATTR_DIMMED, enable);
    SetCtrlAttribute (panelHandle, PANEL_ERROR_TERM,       ATTR_DIMMED, enable);
    SetCtrlAttribute (panelHandle, PANEL_PLOT_ERROR_TERM,  ATTR_DIMMED, enable);
    SetCtrlAttribute (panelHandle, PANEL_SAVE_ERROR_TERMS, ATTR_DIMMED, enable);
    SetCtrlAttribute (panelHandle, PANEL_RECALL_ERROR_TERMS, ATTR_DIMMED, enable);
    SetCtrlAttribute (panelHandle, PANEL_FILE_NAME,        ATTR_DIMMED, enable);
}


/*===========================================================================*/
/* Function: Cleanup
/* Purpose:  Closes the VISA session to the instrument and in case of error
/*           queries the error and shows the dialog box with the passed error
/*           message. In case Device Clear is requested, it is necessary to 
/*           set the argument deviceClear to VI_TRUE.
/*===========================================================================*/
void cleanup (ViBoolean deviceClear)
{

    /* Device Clear is a low-level command and should be used in case the
     * instrument is waiting for Operation Complete to cancel the wait.
     * It is useful for instance in case of incorrect external trigger
     * when the instrument does not respond to any other command because of
     * waiting for trigger.
     */
    if (deviceClear)
    {
        viClear (instrSession);
        viPrintf (instrSession, "*CLS\n");
    }
    
    /* close instrument session */
    rsvna_close (instrSession);
    instrSession = VI_NULL;
    
    /* no measurements */
    enableTrace  = VI_FALSE;
    
    /* enable GUI */
    EnableButtons (1);
    /* enable mouse events*/
    SetWaitCursor (0);
    
}

/*===========================================================================*/
/* Function: saveErrorTerms Callback
/* Purpose:  Every time the Save Error Terms button is pressed all the error
/*           terms are stored to a file on a disk. Then can be later recalled 
/*           from this file by pressing Recall Error Terms button.
/*===========================================================================*/
int CVICALLBACK saveErrorTerms (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViChar file_name[STRING_LENGTH] = "";
    FILE   *targetFile              = NULL;
    ViByte buffer[500*STRING_LENGTH];
    ViByte temp[STRING_LENGTH];
    ViUInt32 bytesReadInstr         = 0,
             bytesWrittenFile       = 0;
    ViInt32  i                      = 0,
             x                      = 0;
    
    switch (event)
    {
        case EVENT_COMMIT:  /* Save Error Terms button pressed */
            
            GetCtrlVal (panelHandle, PANEL_FILE_NAME, file_name);
            
            if ((targetFile = fopen (file_name, "w")) == NULL)
                return RS_ERROR_CANNOT_OPEN_FILE;
            
            for (x = 0; x < 10; x++)
            {
                for (i = 0; i < values_returned[x]; i++)
                {
                    sprintf (temp, "%lf,", amplitude[x][i]);
                    strcat (buffer, temp);
                }
                strcat (buffer, "\n");
            }
            bytesWrittenFile = (ViUInt32) fwrite (buffer, sizeof (ViByte), (size_t)strlen (buffer), targetFile);
            
            (void) fclose (targetFile);

            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: recallErrorTerms Callback
/* Purpose:  Every time the Recall Error Terms button is pressed all the error
/*           terms are recalled from a file on a disk. If there is no file 
/*           saved yet, press Save Error Terms button to create it.
/*===========================================================================*/
int CVICALLBACK recallErrorTerms (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViStatus    status           = VI_SUCCESS;
    ViChar file_name[STRING_LENGTH] = "";
    FILE   *sourceFile              = NULL;
    ViChar buffer[500*STRING_LENGTH];
    ViChar *temp;
    ViUInt32 bytesRead              = 0,
             bytesWrittenFile       = 0;
    ViInt32  i                      = 0,
             x                      = 0;
    ViChar *p2buf;
    long  fileLen;               /* Length of file */
  
    switch (event)
    {
        case EVENT_COMMIT:  /* Recall Error Terms button pressed */
            
            GetCtrlVal (panelHandle, PANEL_FILE_NAME, file_name);
            
            if ((sourceFile = fopen (file_name, "r")) == NULL)
                return RS_ERROR_CANNOT_OPEN_FILE;
            
            fseek(sourceFile, 0L, SEEK_END);  /* Position to end of file */
            fileLen = ftell(sourceFile);     /* Get file length */
            rewind(sourceFile);               /* Back to start of file */
            
            temp = calloc(fileLen + 1, sizeof(ViChar));
            
            fread(temp, fileLen, 1, sourceFile);
            
            (void) fclose (sourceFile);
            
            strcpy (buffer, temp);
            
            p2buf = strtok (buffer, ",");
            
            while (p2buf != NULL && i < 10)
            {
                amplitude[i][x++] = atof (p2buf);
                p2buf = strtok (NULL, ",");
                if (strstr (p2buf, "\n"))
                {
                    values_returned[i] = x;
                    i++;
                    x = 0;
                }
            }
            
            CHECKERR (rsvna_SelectCalibrationType (instrSession, 1, "XYZ", RSVNA_VAL_CALTYPE_TOSM, 1, 2, 0, 0));
                
            /* Generate a set of default correction data, switch on user calibration */
            CHECKERR (rsvna_GenerateDefaultCalibrationData (instrSession, 1));
            
            CHECKERR (rsvna_ConfigureSystemErrorCorrectionState (instrSession, 1, VI_TRUE));
            
            /* !!! Important !!! Stop sweep when loading error terms */
            CHECKERR (rsvna_ConfigureAcquisition (instrSession, 1, RSVNA_VAL_SWEEP_ALL_CHAN, VI_FALSE, 10, RSVNA_VAL_SWEEP_ALL_CHAN));
            
            /* Recall error terms */
            CHECKERR (rsvna_WriteCalibrationData (instrSession, 1, 0, 1, 0, amplitude[0]));
            CHECKERR (rsvna_WriteCalibrationData (instrSession, 1, 1, 1, 0, amplitude[1]));
            CHECKERR (rsvna_WriteCalibrationData (instrSession, 1, 2, 1, 0, amplitude[2]));
            CHECKERR (rsvna_WriteCalibrationData (instrSession, 1, 3, 1, 2, amplitude[3]));
            CHECKERR (rsvna_WriteCalibrationData (instrSession, 1, 4, 1, 2, amplitude[4]));
            CHECKERR (rsvna_WriteCalibrationData (instrSession, 1, 0, 2, 0, amplitude[5]));
            CHECKERR (rsvna_WriteCalibrationData (instrSession, 1, 1, 2, 0, amplitude[6]));
            CHECKERR (rsvna_WriteCalibrationData (instrSession, 1, 2, 2, 0, amplitude[7]));
            CHECKERR (rsvna_WriteCalibrationData (instrSession, 1, 3, 2, 1, amplitude[8]));
            CHECKERR (rsvna_WriteCalibrationData (instrSession, 1, 4, 2, 1, amplitude[9]));
            
            
            break;
    }
    return 0;
}
