#include "rsvna.h"
#include <cvirte.h>     
#include <userint.h>
#include <visa.h>
#include "rsvna_single_sweep_mode_example.h"


#define ARRAY_SIZE    5000
#define STRING_LENGTH 512
#define WINDOW        1

/* Useful macros */
#define CHECKERR(fCal) \
    if (status = checkError((fCal)), status < VI_SUCCESS) \
        return status; else


/* Global variables */
static   ViInt32    panelHandle  = 0;
volatile ViBoolean  enableTrace  = VI_FALSE; 
static   ViSession  instrSession = VI_NULL;
ViReal64            amplitude [ARRAY_SIZE];
    

/* Utility functions declarations */
ViStatus checkError(ViStatus status);
void cleanup (ViBoolean deviceClear);
void EnableButtons(ViBoolean enable);

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)
        return -1;  /* out of memory */
    if ((panelHandle = LoadPanel (0, "rsvna_single_sweep_mode_example.uir", PANEL)) < 0)
        return -1;
    
    DisplayPanel (panelHandle);
    RunUserInterface ();
    DiscardPanel (panelHandle);
    
    return 0;
}

int CVICALLBACK panelCallback (int panel, int event, void *callbackData,
        int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_CLOSE:
            /* cleanup specan driver session */
            cleanup(VI_FALSE);
            
            QuitUserInterface (0);
            break;
    }
    return 0;
}


/*===========================================================================*/
/* Function: Start Button Callback
/* Purpose:  Configures the instrument according to selected measurement
/*           parameters and enables the measurement loop
/*===========================================================================*/
int CVICALLBACK startButton (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViChar      instr_desc [STRING_LENGTH] = "";
    ViReal64    sweep_time       = 0.0;
    ViInt32     sweep_count      = 0;
    ViInt32     id               = 0;
    ViInt32     reset            = VI_FALSE;
    ViInt32     display          = VI_FALSE;
    ViStatus    status           = VI_SUCCESS;
    ViStatus    temp_status  = VI_SUCCESS;
    
    switch (event)
    {
        case EVENT_COMMIT:     /* start button pressed */
        {
        
            /* block mouse events */
            SetWaitCursor (1);
            
            /* read values from panel*/
            GetCtrlVal (panelHandle, PANEL_INSTR_DESC, instr_desc);
            GetCtrlVal (panelHandle, PANEL_IDQUERY, &id);
            GetCtrlVal (panelHandle, PANEL_RESET, &reset);
            GetCtrlVal (panelHandle, PANEL_SWEEP_TIME, &sweep_time);
            GetCtrlVal (panelHandle, PANEL_SWEEP_COUNT, &sweep_count);
            GetCtrlVal (panelHandle, PANEL_CHECKBOX, &display);

            /* deletes all points of the graph plot */
            temp_status = DeleteGraphPlot (panelHandle, PANEL_GRAPH, -1, VAL_DELAYED_DRAW);
            if( temp_status != 0 )
            {
                /* prompt CVI error message */
                MessagePopup ("Error", GetUILErrorString(temp_status));
                
                /* cleanup specan driver session */
                cleanup(VI_FALSE);
                
                /* do not free resource */
                enableTrace = VI_FALSE;
                return -1;
            }
            
            /* disable buttons for configuration */
            EnableButtons(VI_FALSE);
            
            /* initialize driver */
            CHECKERR (rsvna_init (instr_desc, id, reset, &instrSession));
        
            /* switch device display on/off */
            CHECKERR (rsvna_SetAttributeViInt32 (instrSession, "", RSVNA_ATTR_DISPLAY_UPDATE, display));
        
            /* Configure Sweep Time */
            CHECKERR (rsvna_ConfigureSweepTime (instrSession, 1, VI_FALSE, sweep_time));
            
            /* Set instrument to single sweep mode - continous mode is done
             * programmatically because this is the only way how to
             * synchronize measurement using *OPC
             */
            CHECKERR (rsvna_ConfigureAcquisition (instrSession, 1, RSVNA_VAL_SWEEP_ALL_CHAN, VI_FALSE, sweep_count, RSVNA_VAL_SWEEP_ALL_CHAN));
            
            /* Perform Sweep */
            CHECKERR (rsvna_RestartSweep (instrSession, 1, RSVNA_VAL_SWEEP_ALL_CHAN));
        
            /* start measurements */
            enableTrace = VI_TRUE;
            
            /* enable mouse events*/
            SetWaitCursor (0);
        }
        break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Stop Button Callback
/* Purpose:  Stops the continuous measurement, in other cases has no sense
/*===========================================================================*/
int CVICALLBACK stopButton (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
         
    {
        case EVENT_COMMIT:
            
            /* stop measurements */
            enableTrace = VI_FALSE;
            
            /* cleanup specan driver session */
            cleanup(VI_TRUE);
                
            /* enable buttons for configuration */
            EnableButtons(VI_TRUE);
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: runSingleSweep Callback
/* Purpose:  Every time the Run Single Sweep button is pressed the Y-trace is 
/*           read out of the device and plotted as a graph.
/*===========================================================================*/
int CVICALLBACK querySelectedSweep (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViStatus temp_status  = VI_SUCCESS;
    ViInt32  timeout      = 5000;               /* default value */
    ViInt32  actual_count = 0;
    ViChar   error_buffer [1024];
    ViInt32  sweep_number = 0;
    ViInt32  sweep_count  = 0;
    ViReal64    ampl_x[201];
    ViReal64    ampl_y[201];
    int i;
    
    switch (event)
    {
        case EVENT_COMMIT:  /* Query Selected Sweep button pressed */
            
            /* already configured and ready for measurements?*/
            if(enableTrace==VI_FALSE)
                return 0;
            
            /* exclusive execution from now on! */          
            enableTrace = VI_FALSE;

            GetCtrlVal (panelHandle, PANEL_SWEEP_NUMBER, &sweep_number);
            GetCtrlVal (panelHandle, PANEL_SWEEP_COUNT, &sweep_count);
            
            if (sweep_number > sweep_count)
            {
                MessagePopup ("Sweep Number", "Sweep Number To Read is higher than Sweep Count. Select number small or equal to Sweep Count.");
                enableTrace = VI_TRUE;
                return -1;
            }
            
            /* Reads the response values of a trace acquired in single sweep mode. */
            temp_status = rsvna_TraceResponseSingleSweepData (instrSession, 1, sweep_number, ARRAY_SIZE, &actual_count, amplitude);
                
            if( temp_status != VI_SUCCESS )
            {
                /* check error (specan driver) and prompt it */
                checkError(temp_status);
                
                /* do not free resource */
                enableTrace = VI_FALSE;
                return -1;
            }
            
            /* deletes all points of the graph plot */
            temp_status = DeleteGraphPlot (panelHandle, PANEL_GRAPH, -1, VAL_DELAYED_DRAW);
            if( temp_status != 0 )
            {
                /* prompt CVI error message */
                MessagePopup ("Error", GetUILErrorString(temp_status));
                
                /* cleanup specan driver session */
                cleanup(VI_FALSE);
                
                /* do not free resource */
                enableTrace = VI_FALSE;
                return -1;
            }
            
            for (i = 0; i < actual_count/2; i++)
            {
                ampl_x[i] = amplitude[2*i];
                ampl_y[i] = amplitude[2*i+1];
            }
            
            /* Plots a waveform onto a graph control */
            temp_status = PlotXY (panelHandle, PANEL_GRAPH, ampl_x, ampl_y,
                                  actual_count/2, VAL_DOUBLE, VAL_DOUBLE,
                                  VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1,
                                  VAL_RED);
            if( temp_status < 0 )
            {
                /* prompt CVI error message */
                MessagePopup ("Error", GetUILErrorString(temp_status));
                
                /* cleanup specan driver session */
                cleanup(VI_FALSE);
                
                /* do not free resource */
                enableTrace = VI_FALSE;
                return -1;
            }
            
            /* free resource */         
            enableTrace = VI_TRUE;
            break;
    }
    return 0;
}




/*===========================================================================*/
/* Function: checkError
/* Purpose:  If passed status indicates an failure, the error message will
/*           be displayed.
/*===========================================================================*/
ViStatus checkError (ViStatus status)
{
    ViChar  error_message [STRING_LENGTH];
    ViChar  error_buffer  [STRING_LENGTH*2];
    ViChar* p2buf;
    ViInt32 error;
    
    if (status < VI_SUCCESS)
    {
        /* Converts a status code returned by an instrument driver function into a user-readable string */
        rsvna_error_message (instrSession, status, error_message);
        p2buf = error_buffer + sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", status, error_message);
        
        
        /* This function reads an error code and a message from the instrument's error queue */
        rsvna_error_query (instrSession, &error, error_message);
        
        SetWaitCursor (0);
        if (error != VI_SUCCESS)
        {
            sprintf (p2buf, "Secondary Error: 0x%08X, %s\n", error, error_message);
        }
        MessagePopup ("Error", error_buffer);
        
        /* close session */
        cleanup(VI_TRUE);
        
        /* enable buttons for configuration */
        EnableButtons(VI_TRUE);
    }
    return status;
}


/*===========================================================================*/
/* Function: EnableButtons
/* Purpose:  This function dis-/enables GUI while measuring/configuration.
/*===========================================================================*/
void EnableButtons(ViBoolean enable){

    SetCtrlAttribute (panelHandle, PANEL_SWEEP_TIME,       ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_SWEEP_COUNT,      ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_START,            ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_RESET,            ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_IDQUERY,          ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_INSTR_DESC,       ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_CHECKBOX,         ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_STOP,             ATTR_DIMMED, enable);
    SetCtrlAttribute (panelHandle, PANEL_SWEEP_NUMBER,     ATTR_DIMMED, enable);
    SetCtrlAttribute (panelHandle, PANEL_QUERY_SELECTED_SWEEP, ATTR_DIMMED, enable);
}


/*===========================================================================*/
/* Function: Cleanup
/* Purpose:  Closes the VISA session to the instrument and in case of error
/*           queries the error and shows the dialog box with the passed error
/*           message. In case Device Clear is requested, it is necessary to 
/*           set the argument deviceClear to VI_TRUE.
/*===========================================================================*/
void cleanup (ViBoolean deviceClear)
{

    /* Device Clear is a low-level command and should be used in case the
     * instrument is waiting for Operation Complete to cancel the wait.
     * It is useful for instance in case of incorrect external trigger
     * when the instrument does not respond to any other command because of
     * waiting for trigger.
     */
    if (deviceClear)
    {
        viClear (instrSession);
        viPrintf (instrSession, "*CLS\n");
    }
    
    /* close instrument session */
    rsvna_close (instrSession);
    instrSession = VI_NULL;
    
    /* no measurements */
    enableTrace  = VI_FALSE;
    
    /* enable GUI */
    EnableButtons (1);
    /* enable mouse events*/
    SetWaitCursor (0);
    
}
